#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const Koa = require("koa");
const bodyParser = require("koa-bodyparser");
const Router = require("koa-router");
const winston = require("winston");
exports.app = new Koa();
exports.app.proxy = Boolean(process.env.PROXY) || false;
const router = new Router();
const logger = new winston.Logger();
logger.add(winston.transports.File, {
    filename: process.env.LOG_FILE || path_1.join(__dirname, 'request.log'),
    json: process.env.JSON_LOG || true,
    maxFiles: 5,
    maxsize: 5000000,
    tailable: true
});
if (process.env.NODE_ENV !== 'test') {
    logger.add(winston.transports.Console);
}
router.post('/', (ctx, next) => __awaiter(this, void 0, void 0, function* () {
    const body = ctx.request.body;
    if (!ctx.request.body || !body.user_id && !body.userId) {
        return ctx.status = 400;
    }
    yield handleAuthRequest(ctx);
    next();
}));
function handleAuthRequest(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        const request = ctx.request;
        const body = request.body;
        const userId = body.user_id || body.userId;
        const ip = request.ip;
        const blacklisted = yield isUserBlacklisted(userId);
        const response = {
            blacklisted
        };
        if (blacklisted) {
            ctx.status = 401;
        }
        logger.info('request from %s for user %s --> blacklisted %s', ip, userId, blacklisted, {
            blacklisted,
            ip,
            plugin: body.plugin,
            port: request.headers['bukkit-server-port'] || body.port,
            userId
        });
        ctx.set('Content-Type', 'application/json');
        ctx.body = JSON.stringify(response);
    });
}
function isUserBlacklisted(userId) {
    return __awaiter(this, void 0, void 0, function* () {
        const bannedFileLocation = process.env.BLACKLISTED_USERS_FILE || path_1.join(__dirname, 'banned_users.txt');
        const bannedUsersFile = yield readFileAsync(bannedFileLocation);
        if (!bannedUsersFile) {
            return false;
        }
        const bannedUsers = bannedUsersFile.toString().match(/[^\r\n]+/g) || [];
        return bannedUsers.includes(userId);
    });
}
function readFileAsync(file) {
    return new Promise(resolve => {
        fs_1.readFile(file, 'utf8', (err, data) => {
            resolve(data);
        });
    });
}
exports.app.use(bodyParser());
exports.app.use(router.routes());
exports.app.use(router.allowedMethods());
const server = exports.app.listen(process.env.PORT || 3000, () => {
    const loggerFileLocation = process.env.LOG_FILE || path_1.join(__dirname, 'request.log');
    const bannedFileLocation = process.env.BLACKLISTED_USERS_FILE || path_1.join(__dirname, 'banned_users.txt');
    logger.info('Spigot Anti Piracy Backend listening at http://%s:%s', server.address().address, server.address().port);
    logger.info('Logging to %s', loggerFileLocation);
    logger.info('Using %s for blacklisted users', bannedFileLocation);
});
